import * as React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import { parseMetadata } from '@highlight-ui/utils-commons';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

var styles$1 = {"drawerContainer":"Drawer-module__37T18NwR__v2-1-9"};

var Drawer = /** @class */ (function (_super) {
    __extends(Drawer, _super);
    function Drawer(props) {
        var _this = _super.call(this, props) || this;
        _this.determineBodyOverflowStyle = function () {
            // Set overflow hidden in order to prevent body scrolling while
            // the full modal is opened
            var _a = _this.props, disabledBodyScrolling = _a.disabledBodyScrolling, visible = _a.visible;
            if (disabledBodyScrolling) {
                window.document.body.style.overflowY = visible ? 'hidden' : 'unset';
            }
        };
        _this.handleKeyDown = function (event) {
            var _a = _this.props, onRequestToClose = _a.onRequestToClose, closeKey = _a.closeKey, preventToClose = _a.preventToClose;
            if (String(event.keyCode) === String(closeKey) && !preventToClose) {
                onRequestToClose();
            }
        };
        _this.ref = React.createRef();
        return _this;
    }
    Drawer.prototype.componentDidMount = function () {
        // eslint-disable-next-line react/destructuring-assignment
        if (this.props.visible) {
            document.addEventListener('keydown', this.handleKeyDown);
        }
        this.determineBodyOverflowStyle();
    };
    Drawer.prototype.componentDidUpdate = function (prevProps) {
        var _a = this.props, visible = _a.visible, onOpen = _a.onOpen, onClose = _a.onClose;
        if (visible !== prevProps.visible) {
            // Prop callbacks
            if (visible) {
                onOpen();
            }
            else {
                onClose();
            }
            this.determineBodyOverflowStyle();
            // Add / Remove event for closing using keyboard
            if (visible)
                document.addEventListener('keydown', this.handleKeyDown);
            else
                document.removeEventListener('keydown', this.handleKeyDown);
        }
    };
    Drawer.prototype.componentWillUnmount = function () {
        document.removeEventListener('keydown', this.handleKeyDown);
    };
    Object.defineProperty(Drawer.prototype, "sizeStyle", {
        get: function () {
            var _a = this.props, topOffset = _a.topOffset, leftOffset = _a.leftOffset;
            return {
                top: topOffset,
                left: leftOffset,
                width: "calc(100% - " + leftOffset + ")",
                height: "calc(100% - " + topOffset + ")",
            };
        },
        enumerable: false,
        configurable: true
    });
    Drawer.prototype.render = function () {
        var sizeStyle = this.sizeStyle;
        var _a = this.props, visible = _a.visible; _a.disabledBodyScrolling; _a.preventToClose; _a.closeKey; _a.topOffset; _a.leftOffset; var metadata = _a.metadata, className = _a.className, children = _a.children; _a.onOpen; _a.onClose; _a.onRequestToClose; var remainingProps = __rest(_a, ["visible", "disabledBodyScrolling", "preventToClose", "closeKey", "topOffset", "leftOffset", "metadata", "className", "children", "onOpen", "onClose", "onRequestToClose"]);
        var containerClassName = classnames(styles$1.drawerContainer, className);
        var Component = (React.createElement("div", __assign({}, remainingProps, parseMetadata(metadata), { className: containerClassName, style: sizeStyle, "data-state": visible && 'show', ref: this.ref }), children));
        return ReactDOM.createPortal(Component, window.document.body);
    };
    // eslint-disable-next-line react/static-property-placement
    Drawer.defaultProps = {
        visible: false,
        disabledBodyScrolling: true,
        preventToClose: false,
        closeKey: 27,
        topOffset: '0px',
        leftOffset: '280px',
        className: null,
        children: null,
        onOpen: function () { },
        onClose: function () { },
        onRequestToClose: function () { },
    };
    return Drawer;
}(React.PureComponent));

var DrawerBody = React.forwardRef(function (_a, ref) {
    var className = _a.className, children = _a.children, metadata = _a.metadata;
    return (React.createElement("div", __assign({}, parseMetadata(metadata), { className: classnames(className), ref: ref }), children));
});
var DrawerBody$1 = React.memo(DrawerBody);

var styles = {"drawerHeaderContainer":"DrawerHeader-module__610UTmqX__v2-1-9","drawerHeaderContainerFixed":"DrawerHeader-module__1C_oNt0Y__v2-1-9","drawerHeaderContainerFixedWithScroll":"DrawerHeader-module__1jW-TnjL__v2-1-9"};

var DrawerHeader = /** @class */ (function (_super) {
    __extends(DrawerHeader, _super);
    function DrawerHeader() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleRef = function (ref) {
            _this.ref = ref;
            _this.attachEvents();
        };
        _this.attachEvents = function () {
            var fixed = _this.props.fixed;
            if (_this.ref) {
                var parentEl = _this.ref.parentElement;
                // Make sure just to add the event if is the drawer element
                var drawerEl = parentEl && parentEl.getAttribute('role') === 'drawer'
                    ? parentEl
                    : null;
                if (!drawerEl) {
                    return;
                }
                if (fixed) {
                    drawerEl.addEventListener('scroll', _this.onScroll);
                }
                else {
                    drawerEl.removeEventListener('scroll', _this.onScroll);
                }
            }
        };
        _this.onScroll = function (event) {
            var scrollTop = event.currentTarget.scrollTop;
            var appendFixedClass = scrollTop !== 0;
            _this.ref.classList[appendFixedClass ? 'add' : 'remove'](styles.drawerHeaderContainerFixedWithScroll);
        };
        return _this;
    }
    DrawerHeader.prototype.componentUpdate = function (prevProps) {
        var fixed = this.props.fixed;
        if (prevProps.fixed !== fixed)
            this.attachEvents();
    };
    DrawerHeader.prototype.render = function () {
        var _a;
        var _b = this.props, fixed = _b.fixed, className = _b.className, children = _b.children, metadata = _b.metadata;
        var containerClassName = classnames((_a = {}, _a[styles.drawerHeaderContainerFixed] = fixed, _a), styles.drawerHeaderContainer, className);
        return (React.createElement("div", __assign({}, parseMetadata(metadata), { className: containerClassName, ref: this.handleRef }), children));
    };
    // eslint-disable-next-line react/static-property-placement
    DrawerHeader.defaultProps = {
        fixed: false,
        className: null,
    };
    return DrawerHeader;
}(React.PureComponent));

export { Drawer, DrawerBody$1 as DrawerBody, DrawerHeader };
//# sourceMappingURL=index.js.map
